@@ -10,16 +10,17 @@ from collections import defaultdict
 from inspect import getmembers, signature, getsource, isclass
 import modeci_mdf
 import numpy as np
-import sympy
 import torch
 from typing import Union, Dict, Any, Tuple, List, Callable
 import torch.nn as nn
 from modeci_mdf.functions.standard import mdf_functions
 from modeci_mdf.utils import load_mdf
 from modeci_mdf.execution_engine import EvaluableGraph
+from modeci_mdf import __version__
 
 
-not_self = set()
+# Todo: remove these global variables!!
+# not_self = set()
 param_set = set()
 graph_input = []
 
@@ -43,6 +44,8 @@ def get_module_declaration_text(
     """
     declaration_text = ("\n\nclass {}(nn.Module):").format(name)
 
+    not_self = set()
+
     input_ports = node_dict["input_ports"]
     functions = node_dict["functions"]
     parameters = node_dict["parameters"]
@@ -189,6 +192,7 @@ def get_module_declaration_text(
                 for function in functions:
                     declaration_text += f"\n\t\t{function.id}="
                     func_dic = function.function
+                    exp = ""
                     for i in func_dic:
                         if i in mdf_functions:
                             exp = mdf_functions[i]["expression_string"]
@@ -299,17 +303,28 @@ def build_script(
     """
     model_id = model_id1 + ".onnx"
     script = ""
+    base_info = (
+        "'''\nThis script has been generated by modeci_mdf v"
+        + __version__
+        + ".\nIt is an export of a MDF model"
+    )
     if version == "mdf.s":
-        s1 = "Converted examples from MDF models version(mdf.s) to PyTorch/ONNX"
-        script += f'"{s1}"'
+        s1 = (
+            base_info
+            + " (mdf.s - MDF stateful, i.e. full MDF allowing stateful parameters) to PyTorch\n"
+        )
+        script += s1
     elif version == "mdf.0":
-        s2 = "Converted examples from MDF models version(mdf.0) to PyTorch/ONNX"
-        script += f'"{s2}"'
+        s2 = base_info + " (mdf.0  - MDF zero, a simplified form of MDF) to PyTorch\n"
+        script += s2
+
+    script += "\n'''\n"
+
     imports_string = (
         "\nimport torch"
         "\nimport torch.nn as nn\nimport onnx\nimport onnxruntime as rt\nfrom math import *"
     )
-    print(script)
+    # print(script)
 
     # Declarations string
     modules_declaration_text = ""
@@ -354,7 +369,7 @@ def build_script(
     if len(nodes) == 1:
         script += "f\nmodel={nodes[0].id}"
         return script
-    script += "\nmodel = Model("
+    script += "\n\nmodel = Model("
     for node in nodes:
         script += f"{node.id}={node.id}(),"
     script += ")"
@@ -368,9 +383,9 @@ def build_script(
     script += f"\nonnx_model = onnx.load('{model_id}')"
     script += "\nonnx.checker.check_model(onnx_model)"
     script += f"\nsess = rt.InferenceSession('{model_id}')"
-    script += "\nres = sess.run(None, {sess.get_inputs()[0].name: dummy_input.numpy()})"
+    script += "\nres = sess.run(None, {sess.get_inputs()[0].name: dummy_input.numpy()} if len(sess.get_inputs())>0 else {})"
     x, y = "__main__", "Exported to PyTorch and ONNX"
-    script += f"\nif __name__ == '{x}':"
+    script += f"\n\nif __name__ == '{x}':"
     script += f"\n\tprint('{y}')"
 
     return script
@@ -442,7 +457,7 @@ def _script_to_model(script: str, model_id1: str, version: str, model_input: str
     """
     import importlib.util
 
-    print("version", version)
+    # print("version", version)
     if version == "mdf.s":
         path_list = model_input.split("/")[:-2] + ["PyTorch/MDF_PyTorch/"]
         out_filename = "/".join(path_list)
@@ -468,7 +483,8 @@ def mdf_to_pytorch(
     mdf_model: "Model", model_input: str, eval_models: bool, version: str
 ):
     """Function loads and returns a pytorch model for all models specified in an
-    mdf file
+    mdf file.
+
     Args:
         mdf_model: model in MDF format
         eval_models: Set Evaluation of model to True or False
@@ -481,6 +497,8 @@ def mdf_to_pytorch(
     )
     scripts = _generate_scripts_from_model(mdf_model, version)
     models = {}
+    param_set.clear()
+    graph_input.clear()
 
     for script_name, script in scripts.items():
         model = _script_to_model(script, mdf_model.id, version, model_input)
